package com.foogod.btroutefix;

import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;

public class RouteFixService extends Service {
	public static final String LOG_TAG = "BTRouteFix";
	
	public static final int MSG_FIX_ROUTING = 0;
	
	boolean mHeadsetPlugged = false;
	
	private BroadcastReceiver mReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if (Intent.ACTION_HEADSET_PLUG.equals(intent.getAction())) {
				int state = intent.getIntExtra("state", 0);
				mHeadsetPlugged = state > 0;
			}
		}
	};

	private PhoneStateListener mPhoneStateListener = new PhoneStateListener() {
		@Override
		public void onCallStateChanged(int state, String incoming_number) {
			if (state == TelephonyManager.CALL_STATE_IDLE) {
				mHandler.sendEmptyMessageDelayed(MSG_FIX_ROUTING, 100);
			}
        }
	};
	
	private Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MSG_FIX_ROUTING:
                AudioManager am = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
                int current_routing = am.getRouting(AudioManager.MODE_NORMAL);
                if ((current_routing & AudioManager.ROUTE_BLUETOOTH_SCO) == 0) {
                	if (mHeadsetPlugged) {
                        int normal_routing = AudioManager.ROUTE_HEADSET;
                        int ringtone_routing = AudioManager.ROUTE_HEADSET | AudioManager.ROUTE_SPEAKER;
                        int incall_routing = AudioManager.ROUTE_HEADSET;
                        int rmask = AudioManager.ROUTE_ALL & ~AudioManager.ROUTE_BLUETOOTH_A2DP;
                        Log.i(LOG_TAG, "Bluetooth routing ended.  Resetting correct headset routing ("+current_routing+"-->"+normal_routing+")");
                        am.setRouting(AudioManager.MODE_NORMAL,   normal_routing, rmask);
                        am.setRouting(AudioManager.MODE_RINGTONE, ringtone_routing, rmask);
                        am.setRouting(AudioManager.MODE_IN_CALL,  incall_routing, rmask);
                	}
                } else {
                	// Hasn't happened yet, so we'll have to call ourselves again later..
                	sendEmptyMessageDelayed(MSG_FIX_ROUTING, 100);
                }
                break;
			}
		}
	};
	
	@Override
	public void onStart(Intent intent, int startid) {
        Log.i(LOG_TAG, "Service starting...");
		super.onStart(intent, startid);
        IntentFilter f = new IntentFilter();
        f.addAction(Intent.ACTION_HEADSET_PLUG);
        registerReceiver(mReceiver, f);
		TelephonyManager tm = (TelephonyManager)getSystemService(TELEPHONY_SERVICE);
		tm.listen(mPhoneStateListener, PhoneStateListener.LISTEN_CALL_STATE);
	}
	@Override
	public void onDestroy() {
		// This should never actually be called, but do things correctly anyway.
        Log.i(LOG_TAG, "Service stopped.");
		super.onDestroy();
		TelephonyManager tm = (TelephonyManager)getSystemService(TELEPHONY_SERVICE);
		tm.listen(mPhoneStateListener, PhoneStateListener.LISTEN_NONE);
		unregisterReceiver(mReceiver);
	}
	@Override
	public IBinder onBind(Intent intent) {
		// Nothing should ever need to bind to this.
		return null;
	}

}
